const path = require('node:path')
const process = require('node:process')

const DEFAULT_CONFIG = {
  languages: ['typescript', 'javascript', 'json', 'html', 'css', 'php'],
  themes: ['github-light', 'github-dark'],
}

function validateConfig(config) {
  const errors = []

  if (!Array.isArray(config.languages)) {
    errors.push('languages must be an array')
  }
  if (!Array.isArray(config.themes)) {
    errors.push('themes must be an array')
  }

  if (config.languages?.length) {
    config.languages.forEach((lang) => {
      if (typeof lang !== 'string') {
        errors.push(`Invalid language: ${lang}. Must be a string.`)
      }
    })
  }

  if (config.themes?.length) {
    config.themes.forEach((theme) => {
      if (typeof theme !== 'string') {
        errors.push(`Invalid theme: ${theme}. Must be a string.`)
      }
    })
  }

  if (errors.length > 0) {
    throw new Error(`Invalid rn-shiki config:\n${errors.join('\n')}`)
  }
}

function mergeConfigs(userConfig = {}) {
  return {
    languages: [...new Set([...DEFAULT_CONFIG.languages, ...(userConfig.languages || [])])],
    themes: [...new Set([...DEFAULT_CONFIG.themes, ...(userConfig.themes || [])])],
  }
}

function loadProjectConfig() {
  try {
    const configPath = path.resolve(process.cwd(), 'rn-shiki.config.js')
    return require(configPath)
  }
  catch (error) {
    if (error.code !== 'MODULE_NOT_FOUND') {
      console.warn('Error loading rn-shiki.config.js:', error.message)
    }
    return {}
  }
}

function createResourceImport(t, type, name) {
  return t.objectProperty(
    t.stringLiteral(name),
    t.callExpression(
      t.identifier('require'),
      [t.stringLiteral(`shiki/${type}/${name}.mjs`)],
    ),
  )
}

module.exports = function (babel) {
  const { types: t } = babel

  return {
    name: 'rn-shiki-transform',
    visitor: {
      ImportDeclaration(path, state) {
        // Merge configurations with defaults
        const config = mergeConfigs({
          ...loadProjectConfig(),
          ...(state.opts?.shiki || state.opts || {}),
        })

        try {
          validateConfig(config)
        }
        catch (error) {
          throw path.buildCodeFrameError(error.message)
        }

        // Handle direct shiki resource imports
        if (path.node.source.value.startsWith('shiki/')) {
          const match = path.node.source.value.match(/shiki\/(langs|themes)\/(.+)\.mjs/)
          if (match) {
            const [, type, name] = match
            const configKey = type === 'langs' ? 'languages' : 'themes'

            if (!config[configKey].includes(name)) {
              throw path.buildCodeFrameError(
                `${type === 'langs' ? 'Language' : 'Theme'} "${name}" is not configured.\n`
                + `Configure it in rn-shiki.config.js or babel plugin options.\n`
                + `Available ${configKey}: ${config[configKey].join(', ')}`,
              )
            }
          }
        }

        // Handle shiki package imports
        if (path.node.source.value === 'shiki') {
          // Transform bundledLanguages
          const langSpec = path.node.specifiers.find(
            spec => spec.imported?.name === 'bundledLanguages'
              || spec.local?.name === 'bundledLanguages',
          )

          if (langSpec) {
            const languagesObject = t.objectExpression(
              config.languages.map(lang => createResourceImport(t, 'langs', lang)),
            )

            const binding = path.scope.getBinding(langSpec.local.name)
            if (binding) {
              binding.referencePaths.forEach((refPath) => {
                refPath.replaceWith(languagesObject)
              })
            }
          }

          // Transform bundledThemes
          const themeSpec = path.node.specifiers.find(
            spec => spec.imported?.name === 'bundledThemes'
              || spec.local?.name === 'bundledThemes',
          )

          if (themeSpec) {
            const themesObject = t.objectExpression(
              config.themes.map(theme => createResourceImport(t, 'themes', theme)),
            )

            const binding = path.scope.getBinding(themeSpec.local.name)
            if (binding) {
              binding.referencePaths.forEach((refPath) => {
                refPath.replaceWith(themesObject)
              })
            }
          }
        }
      },
    },
  }
}
