"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createHighlighter = createHighlighter;
exports.loadLanguage = loadLanguage;
exports.loadTheme = loadTheme;
exports.processTokens = processTokens;
var _core = require("shiki/core");
var _engineJavascript = require("shiki/engine-javascript.mjs");
var _css = _interopRequireDefault(require("shiki/langs/css.mjs"));
var _html = _interopRequireDefault(require("shiki/langs/html.mjs"));
var _javascript = _interopRequireDefault(require("shiki/langs/javascript.mjs"));
var _json = _interopRequireDefault(require("shiki/langs/json.mjs"));
var _php = _interopRequireDefault(require("shiki/langs/php.mjs"));
var _typescript = _interopRequireDefault(require("shiki/langs/typescript.mjs"));
var _githubDark = _interopRequireDefault(require("shiki/themes/github-dark.mjs"));
var _githubLight = _interopRequireDefault(require("shiki/themes/github-light.mjs"));
function _interopRequireDefault(e) { return e && e.__esModule ? e : { default: e }; }
// Import languages

// Import themes

// Define supported languages and themes explicitly

// Bundled languages mapping - using the imported grammars directly
const LANGUAGE_IMPORTS = {
  css: _css.default,
  html: _html.default,
  javascript: _javascript.default,
  json: _json.default,
  php: _php.default,
  typescript: _typescript.default
};

// Bundled themes mapping
const THEME_IMPORTS = {
  'github-dark': _githubDark.default,
  'github-light': _githubLight.default
};

// Singleton highlighter instance with tracking

let highlighterInstance = null;
async function createHighlighter({
  langs,
  themes
}) {
  if (!langs || !themes) {
    throw new Error('Please provide both `langs` and `themes` when creating a highlighter.');
  }

  // Return existing instance if already initialized
  if (highlighterInstance?.highlighter) {
    // Load any new languages that weren't previously loaded
    for (const lang of langs) {
      if (!highlighterInstance.loadedLanguages.has(lang)) {
        const langData = LANGUAGE_IMPORTS[lang];
        await highlighterInstance.highlighter.loadLanguage(langData);
        highlighterInstance.loadedLanguages.add(lang);
      }
    }

    // Load any new themes that weren't previously loaded
    for (const theme of themes) {
      if (!highlighterInstance.loadedThemes.has(theme)) {
        const themeData = THEME_IMPORTS[theme];
        await highlighterInstance.highlighter.loadTheme(themeData);
        highlighterInstance.loadedThemes.add(theme);
      }
    }
    return highlighterInstance.highlighter;
  }
  try {
    // Create highlighter with initial languages and themes
    const highlighter = await (0, _core.createHighlighterCore)({
      engine: (0, _engineJavascript.createJavaScriptRegexEngine)(),
      langs: langs.map(lang => LANGUAGE_IMPORTS[lang]),
      themes: themes.map(theme => THEME_IMPORTS[theme])
    });

    // Store the instance with tracking sets
    highlighterInstance = {
      highlighter,
      loadedLanguages: new Set(langs),
      loadedThemes: new Set(themes)
    };
    return highlighter;
  } catch (error) {
    console.error('Failed to create highlighter:', error);
    throw error;
  }
}

// Function to ensure a theme is loaded
async function loadTheme(theme) {
  if (!highlighterInstance?.highlighter) {
    throw new Error('Highlighter not initialized');
  }
  if (highlighterInstance.loadedThemes.has(theme)) {
    return;
  }
  const themeData = THEME_IMPORTS[theme];
  await highlighterInstance.highlighter.loadTheme(themeData);
  highlighterInstance.loadedThemes.add(theme);
}

// Function to ensure a language is loaded
async function loadLanguage(lang) {
  if (!highlighterInstance?.highlighter) {
    throw new Error('Highlighter not initialized');
  }
  if (highlighterInstance.loadedLanguages.has(lang)) {
    return;
  }
  const langData = LANGUAGE_IMPORTS[lang];
  await highlighterInstance.highlighter.loadLanguage(langData);
  highlighterInstance.loadedLanguages.add(lang);
}
function processTokens(tokens) {
  return tokens.map(token => ({
    content: token.content,
    color: token.color || '#000000',
    fontStyle: token.fontStyle?.toString()
  }));
}
//# sourceMappingURL=index.js.map