"use strict";

import { createHighlighterCore } from 'shiki/core';
import { createJavaScriptRegexEngine } from 'shiki/engine-javascript.mjs';

// Import languages
import css from 'shiki/langs/css.mjs';
import html from 'shiki/langs/html.mjs';
import javascript from 'shiki/langs/javascript.mjs';
import json from 'shiki/langs/json.mjs';
import php from 'shiki/langs/php.mjs';
import typescript from 'shiki/langs/typescript.mjs';

// Import themes
import githubDark from 'shiki/themes/github-dark.mjs';
import githubLight from 'shiki/themes/github-light.mjs';

// Define supported languages and themes explicitly

// Bundled languages mapping - using the imported grammars directly
const LANGUAGE_IMPORTS = {
  css,
  html,
  javascript,
  json,
  php,
  typescript
};

// Bundled themes mapping
const THEME_IMPORTS = {
  'github-dark': githubDark,
  'github-light': githubLight
};

// Singleton highlighter instance with tracking

let highlighterInstance = null;
export async function createHighlighter({
  langs,
  themes
}) {
  if (!langs || !themes) {
    throw new Error('Please provide both `langs` and `themes` when creating a highlighter.');
  }

  // Return existing instance if already initialized
  if (highlighterInstance?.highlighter) {
    // Load any new languages that weren't previously loaded
    for (const lang of langs) {
      if (!highlighterInstance.loadedLanguages.has(lang)) {
        const langData = LANGUAGE_IMPORTS[lang];
        await highlighterInstance.highlighter.loadLanguage(langData);
        highlighterInstance.loadedLanguages.add(lang);
      }
    }

    // Load any new themes that weren't previously loaded
    for (const theme of themes) {
      if (!highlighterInstance.loadedThemes.has(theme)) {
        const themeData = THEME_IMPORTS[theme];
        await highlighterInstance.highlighter.loadTheme(themeData);
        highlighterInstance.loadedThemes.add(theme);
      }
    }
    return highlighterInstance.highlighter;
  }
  try {
    // Create highlighter with initial languages and themes
    const highlighter = await createHighlighterCore({
      engine: createJavaScriptRegexEngine(),
      langs: langs.map(lang => LANGUAGE_IMPORTS[lang]),
      themes: themes.map(theme => THEME_IMPORTS[theme])
    });

    // Store the instance with tracking sets
    highlighterInstance = {
      highlighter,
      loadedLanguages: new Set(langs),
      loadedThemes: new Set(themes)
    };
    return highlighter;
  } catch (error) {
    console.error('Failed to create highlighter:', error);
    throw error;
  }
}

// Function to ensure a theme is loaded
export async function loadTheme(theme) {
  if (!highlighterInstance?.highlighter) {
    throw new Error('Highlighter not initialized');
  }
  if (highlighterInstance.loadedThemes.has(theme)) {
    return;
  }
  const themeData = THEME_IMPORTS[theme];
  await highlighterInstance.highlighter.loadTheme(themeData);
  highlighterInstance.loadedThemes.add(theme);
}

// Function to ensure a language is loaded
export async function loadLanguage(lang) {
  if (!highlighterInstance?.highlighter) {
    throw new Error('Highlighter not initialized');
  }
  if (highlighterInstance.loadedLanguages.has(lang)) {
    return;
  }
  const langData = LANGUAGE_IMPORTS[lang];
  await highlighterInstance.highlighter.loadLanguage(langData);
  highlighterInstance.loadedLanguages.add(lang);
}
export function processTokens(tokens) {
  return tokens.map(token => ({
    content: token.content,
    color: token.color || '#000000',
    fontStyle: token.fontStyle?.toString()
  }));
}
//# sourceMappingURL=index.js.map