import type { BundledLanguage, BundledTheme, ThemedToken } from 'shiki'
import type { SupportedLanguage, SupportedTheme } from '../syntax/highlighter'
import { useEffect, useState } from 'react'
import { createHighlighter } from '../syntax'

interface ShikiResult {
  tokens: ThemedToken[][]
  bg: string
  fg: string
}

export function useSyntaxHighlighter({ text, lang, theme }: { text: string, lang: BundledLanguage, theme: BundledTheme }) {
  const [tokens, setTokens] = useState<ThemedToken[][]>([])
  const [error, setError] = useState<string>()
  const [isLoading, setIsLoading] = useState(true)

  useEffect(() => {
    let mounted = true
    setIsLoading(true)

    async function highlight() {
      if (!text) {
        setIsLoading(false)
        setTokens([])
        return
      }

      try {
        const highlighter = await createHighlighter({
          langs: [lang as SupportedLanguage],
          themes: [theme as SupportedTheme],
        })

        const result = highlighter.codeToTokens(text, {
          lang,
          theme,
        }) as ShikiResult

        if (mounted && result.tokens) {
          setTokens(result.tokens)
          setError(undefined)
        }
      }
      catch (err) {
        console.error('Highlighting error:', err)
        if (mounted) {
          setError(err instanceof Error ? err.message : 'Syntax highlighting failed')
          setTokens([])
        }
      }
      finally {
        if (mounted) {
          setIsLoading(false)
        }
      }
    }

    highlight()
    return () => {
      mounted = false
    }
  }, [text, lang, theme])

  return { tokens, error, isLoading }
}
